from ..presentation import present
import random
from datetime import date
import string
#this test suite assumes the tiny database provided by the dummy.xlsx file
# with two tables the parts and users table


def basic_setup():
    app = present('sqlite:///dummy.db', 'localhost')
    app.config.from_object('config')
    app = app.test_client()
    return app


def test_present_logged_out():
    """Test that a user can't access the application if she hasn't
    logged in yet"""

    global FORBIDDEN_STRING
    app = basic_setup()
    rv = app.get("/")
    assert 'Conjuring up dummy.db' in rv.data
    #make sure you start out as logged out
    assert 'You are currently logged in as Nobody' in rv.data
    #check that all the expected pages are inaccessible
    assert 'Tables in this database' not in rv.data
    assert 'parts' not in rv.data
    routes = ['/parts_view', '/parts_view_page/1', '/parts_view_id/1',
              '/parts_filter', '/parts_filter_results/1',
              '/parts_delete_id/1',
              '/parts_create',
              '/parts_modify_id/1',
              '/users_view', '/users_view_page/1', '/users_view_id/1',
              '/users_filter', '/users_filter_results/1',
              '/users_delete_id/1',
              '/users_create',
              '/users_modify_id/1',
              ]
    for route in routes:
        rv = app.get(route)
        assert "401 Unauthorized" in rv.data


def test_logged_in_wrong_email():
    app = present('sqlite:///dummy.db', 'localhost')
    app.config.from_object('config')
    with app.test_client() as c:
        with c.session_transaction() as sess:
            #fake login wrong email
            sess['email'] = 'abc@example.com'
        rv = c.get("/")
        assert "You are currently logged in as abc@example.com" in rv.data
        assert "email isn't in our list of authorized users." in rv.data
        assert 'Tables in this database' not in rv.data


def test_logged_in():
    """Test that the app shows that the user is logged in"""
    app = present('sqlite:///dummy.db', 'localhost')
    app.config.from_object('config')
    with app.test_client() as c:
        with c.session_transaction() as sess:
            #fake login
            sess.update({'email': 'mr@example.com'})
            sess.update({'logged_in': True})
        rv = c.get("/")
        assert "You are currently logged in as mr@example.com" in rv.data
        assert 'Tables in this database' in rv.data
        assert 'parts' in rv.data
        assert 'users' in rv.data


def test_basic_id_view():
    """Test that the app can show one item"""
    app = present('sqlite:///dummy.db', 'localhost')
    app.config.from_object('config')
    with app.test_client() as c:
        with c.session_transaction() as sess:
            #fake login
            sess.update({'email': 'mr@example.com'})
            sess.update({'logged_in': True})
        rv = c.get("/parts_view_id/2")
        assert 'Id:2' in rv.data
        assert 'Blue bike' in rv.data
        assert 'Lkj-3432' in rv.data


def test_table_view():
    """Test that the app can show one first page of a table"""
    app = present('sqlite:///dummy.db', 'localhost')
    app.config.from_object('config')
    with app.test_client() as c:
        with c.session_transaction() as sess:
            #fake login
            sess.update({'email': 'mr@mr.com'})
            sess.update({'logged_in': True})
        rv = c.get("/parts_view")
        assert 'Table: parts' in rv.data
        for item in ['Blue bike', 'White Airplane', 'Green Submarine',
                     'Amazing Star asteroid', 'A sequence of bricks']:
            assert item in rv.data


def test_filter():
    """Test basic filter functionality"""
    app = present('sqlite:///dummy.db', 'localhost')
    app.config.from_object('config')
    with app.test_client() as c:
        with c.session_transaction() as sess:
            #fake login
            sess.update({'email': 'mr@mr.com'})
            sess.update({'logged_in': True})
        rv = c.get("/parts_filter")
        assert 'Blue bike' not in rv.data
        rv = c.get("/parts_filter?eqid=2&gtid=&ltid=&part_number=&description=&eqprice=&gtprice=1000&ltprice=&eqcount=&gtcount=&ltcount=&eqtop_speed=&gttop_speed=&lttop_speed=&eqdate_added=&gtdate_added=&ltdate_added=")
        assert 'Blue bike' in rv.data
        rv = c.get("/parts_filter?eqid=&gtid=&ltid=&part_number=Kjl-2324&description=&eqprice=&gtprice=&ltprice=&eqcount=&gtcount=&ltcount=&eqtop_speed=&gttop_speed=&lttop_speed=&eqdate_added=&gtdate_added=&ltdate_added=")
        assert 'Blue bike' not in rv.data
        assert 'White Airplane' in rv.data
        rv = c.get("/parts_filter?eqid=&gtid=&ltid=&part_number=&description=asteroid&eqprice=&gtprice=&ltprice=&eqcount=&gtcount=&ltcount=&eqtop_speed=&gttop_speed=&lttop_speed=&eqdate_added=&gtdate_added=&ltdate_added=")
        assert 'White Airplane' not in rv.data
        assert 'Amazing Star asteroid' in rv.data
        rv = c.get("/parts_filter?eqid=&gtid=&ltid=&part_number=&description=&eqprice=&gtprice=&ltprice=&eqcount=&gtcount=&ltcount=&eqtop_speed=&gttop_speed=&lttop_speed=&eqdate_added=&gtdate_added=&ltdate_added=11%2F12%2F2012")
        assert 'Amazing Star asteroid' not in rv.data
        assert 'Green Submarine' in rv.data
        rv = c.get("/parts_filter?eqid=&gtid=&ltid=&part_number=&description=ne&eqprice=&gtprice=&ltprice=&eqcount=&gtcount=&ltcount=&eqtop_speed=&gttop_speed=&lttop_speed=&eqdate_added=&gtdate_added=&ltdate_added=")
        assert 'White Airplane'  in rv.data
        assert 'Amazing Star asteroid' not in rv.data


def test_create_delete():
    """Test add and remove features"""
    app = present('sqlite:///dummy.db', 'localhost')
    app.config.from_object('config')
    with app.test_client() as c:
        with c.session_transaction() as sess:
            #fake login
            sess.update({'email': 'mr@mr.com'})
            sess.update({'logged_in': True})
        rv = c.get("/parts_view_id/2")
        part_number = ''.join(random.choice(string.ascii_letters
                                     + string.digits) for x in range(10))
        description = "Super specific string that should not be a description."
        price = random.random()
        count = random.choice(range(0,100))
        top_speed = random.random() * 100.0
        d = date.today().strftime("%m/%d/%Y")
        csrf_token = u"12345"
        with c.session_transaction() as sess:
            #fake login
            sess.update({'_csrf_token': u'12345'})
        rv = c.post('/parts_create',data=dict(part_number = part_number,
                                              description = description,
                                              price = price,
                                              count = count,
                                              top_speed = top_speed,
                                              date_added = d,
                                              _csrf_token = csrf_token
                                                ),
                                    follow_redirects=True)
        rv = c.get("/parts_view_id/7")
        assert description in rv.data
        with c.session_transaction() as sess:
            #fake login
            sess.update({'_csrf_token': u'12345'})
        rv = c.post("/parts_delete_id/7", data = dict(delete = 'delete_item',
                                                      _csrf_token = csrf_token,
                                                      id = 7), follow_redirects=True)
        rv = c.get("/parts_view_id/7")
        assert "No item with that id." in rv.data
